
#include <windows.h>
#include <imm.h>
#include "vim.h"
#include "winime.h"

extern void DisplayCompString(char_u *s,int len);


static BOOL bKoreanMode = FALSE;
extern HWND s_hwnd;
static char lpCompStr[100];         // Pointer to composition str.
static BOOL bInComposition=FALSE;
static BOOL bCommandMode=TRUE;
static BOOL bImeKorean=FALSE;


void ImeSetOriginMode(void)
{
    HIMC    hImc;
    DWORD   dwConvMode, dwSentMode;
    if ( (  hImc = ImmGetContext( s_hwnd ) ) ) {
        ImmGetConversionStatus(hImc, &dwConvMode, &dwSentMode);
        if ( !(dwConvMode & IME_CMODE_NATIVE) && bKoreanMode == TRUE) {
            ImmSimulateHotKey(s_hwnd, IME_KHOTKEY_ENGLISH);
        }
        else
            bKoreanMode = FALSE;
    }
    bCommandMode = FALSE;
}

void ImeSetEnglishMode(void)
{

    HIMC    hImc;
    DWORD   dwConvMode, dwSentMode;
    if ( (  hImc = ImmGetContext( s_hwnd ) ) ) {
        ImmGetConversionStatus(hImc, &dwConvMode, &dwSentMode);
        if ( dwConvMode & IME_CMODE_NATIVE ) {
            ImmSimulateHotKey(s_hwnd, IME_KHOTKEY_ENGLISH);
            bKoreanMode = TRUE;
        }
        else
            bKoreanMode = FALSE;
    }
    bCommandMode = TRUE;
}

void GetCompositionStr( HWND hwnd, LPARAM CompFlag )
{
     DWORD      dwBufLen;               // Stogare for len. of composition str

     HIMC       hIMC;                   // Input context handle.

     //
     // If fail to get input context handle then do nothing.
     // Applications should call ImmGetContext API to get
     // input context handle.
     //

     if ( !( hIMC = ImmGetContext( hwnd ) ) )
         return;

     //
     // Determines how much memory space to store the composition string.
     // Applications should call ImmGetCompositionString with
     // GCS_COMPSTR flag on, buffer length zero, to get the bullfer
     // length.
     //

     if ( ( dwBufLen = ImmGetCompositionString( hIMC, GCS_COMPSTR, 
                                         (void FAR*)NULL, 0l ) ) < 0 )
         goto exit2;

     if ( dwBufLen > 99 )
         goto exit2;
     //
     // Reads in the composition string.
     //
     if ( dwBufLen != 0 ) {
         ImmGetCompositionString( hIMC, GCS_COMPSTR, lpCompStr, dwBufLen );
         lpCompStr[ dwBufLen ] = 0;
     }
     else {
         strcpy(lpCompStr,"  ");
         dwBufLen = 2;
     }
     
     //
     // Display new composition chars.
     //
     DisplayCompString(lpCompStr, dwBufLen);
    

exit2:

     ImmReleaseContext( hwnd, hIMC );

}


//***********************************************************************
//
// void GetResultStr()
//
// This handles WM_IME_COMPOSITION with GCS_RESULTSTR flag on.
//
//***********************************************************************

void GetResultStr( HWND hwnd )
{
    DWORD       dwBufLen;               // Storage for length of result str.
    HIMC        hIMC;                   // Input context handle.

    //
    // If fail to get input context handle then do nothing.
    //

    if ( !( hIMC = ImmGetContext( hwnd ) ) )
        return;

    //
    // Determines how much memory space to store the result string.
    // Applications should call ImmGetCompositionString with
    // GCS_RESULTSTR flag on, buffer length zero, to get the bullfer
    // length.
    //

    if ( ( dwBufLen = ImmGetCompositionString( hIMC, GCS_RESULTSTR,
                                  (void FAR *)NULL, (DWORD) 0 ) ) <= 0 )
        goto exit2;

    if ( dwBufLen > 99 )
        goto exit2;
    //
    // Reads in the result string.
    //

    ImmGetCompositionString( hIMC, GCS_RESULTSTR, lpCompStr, dwBufLen );

    //
    // Displays the result string.
    //

    DisplayCompString(lpCompStr, dwBufLen);


exit2:

    ImmReleaseContext( hwnd, hIMC );

}

void ImeUIStartComposition(HWND hwnd)
{
     bInComposition = TRUE;
     //GetResultStr( hwnd );
}

void ImeUIComposition(HWND hwnd, WPARAM wParam,LPARAM CompFlag)
{
    
    if ( CompFlag & GCS_RESULTSTR ) {
        GetResultStr( hwnd );
    }
    else if ( CompFlag & GCS_COMPSTR ) {
        GetCompositionStr( hwnd, CompFlag );
    }

}

void ImeUIEndComposition(HWND hwnd)
{
    bInComposition = FALSE;
    //GetResultStr( hwnd );
}


char* ImeGetTempComposition(void)
{
    if ( bInComposition == TRUE /* && bCommandMode == FALSE */) {
        HIMC    hImc;
        DWORD   dwConvMode, dwSentMode;
        if ( (  hImc = ImmGetContext( s_hwnd ) ) ) {
            ImmGetConversionStatus(hImc, &dwConvMode, &dwSentMode);
            if ( (dwConvMode & IME_CMODE_NATIVE) )
                return lpCompStr;
        }
    }
    return NULL;
}

void ImeNotify(WPARAM w, LPARAM l)
{
    HIMC    hImc;
    DWORD   dwConvMode, dwSentMode;
    if ( (  hImc = ImmGetContext( s_hwnd ) ) ) {
        ImmGetConversionStatus(hImc, &dwConvMode, &dwSentMode);
        if ( dwConvMode & IME_CMODE_NATIVE)
            bImeKorean = TRUE;
        else
            bImeKorean = FALSE;
    }
}

void ImeOpenClose(HWND hWnd, BOOL fFlag)
{
    HIMC            hIMC;

    //
    // If fFlag is true then open IME; otherwise close it.
    //

    if ( !( hIMC = ImmGetContext( hWnd ) ) )
        return;

    ImmSetOpenStatus( hIMC, fFlag );

    ImmReleaseContext( hWnd, hIMC );
}

void ImeInit(void)
{
/*    
    static HKL hkl;
    HFONT hFont;
    LOGFONT    lFont;

    if ( ( hFont = (HFONT)SendMessage( s_hwnd, WM_GETFONT, 0, 0L ) ) != NULL ) {
        if ( GetObject( hFont, sizeof(LOGFONT), (LPVOID)&lFont ) ) {
            HIMC	hImc;
            if ( (  hImc = ImmGetContext( s_hwnd ) ) ) {
		 ImmSetCompositionFont( hImc, &lFont );
                 ImmReleaseContext( s_hwnd, hImc );
            }
        }
    }

    ImmGetProperty( GetKeyboardLayout(0),IGP_PROPERTY );


    hkl = GetKeyboardLayout(0L);
 */   
}


/************************************************************************
*
*   IsDBCSTrailByte - returns TRUE if the given byte is a DBCS trail byte
*
*                     The algorithm searchs backward in the string, to some
*                     known character boundary, counting consecutive bytes
*                     in the lead byte range. An odd number indicates the
*                     current byte is part of a two byte character code.
*
*   INPUT: PCHAR  - pointer to a preceding known character boundary.
*          PCHAR  - pointer to the character to test.
*
*   OUTPUT:BOOL   - indicating truth of p==trailbyte.
*
************************************************************************/

int IsDBCSTrailByte( char *base, char *p )
{
    int lbc = 0;    // lead byte count

    if ( base > p )
        return 0;
    if ( strlen(base) <= (size_t)(p-base) )
        return 0;

    while ( p > base ) {
	if ( !IsDBCSLeadByte(*(--p)) )
	    break;
	lbc++;
    }

    return (lbc & 1);
}
